@@ -31,28 +31,24 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
 import os.path
 import threading
-import gobject
+import pkg_resources
+from twisted.internet import reactor
+from twisted.internet.task import LoopingCall
 
-try:
-    import deluge.libtorrent as lt
-except ImportError:
-    import libtorrent as lt
-    if not (lt.version_major == 0 and lt.version_minor == 14):
-        raise ImportError("This version of Deluge requires libtorrent 0.14!")
+from deluge._libtorrent import lt
 
+from deluge.event import *
 import deluge.configmanager
 import deluge.common
 import deluge.component as component
 from deluge.log import LOG as log
 
 DEFAULT_PREFS = {
-    "config_location": deluge.configmanager.get_config_dir(),
     "send_info": False,
     "info_sent": 0.0,
     "daemon_port": 58846,
@@ -60,10 +56,11 @@ DEFAULT_PREFS = {
     "compact_allocation": False,
     "download_location": deluge.common.get_default_download_dir(),
     "listen_ports": [6881, 6891],
+    "listen_interface": "",
     "copy_torrent_file": False,
+    "del_copy_torrent_file": False,
     "torrentfiles_location": deluge.common.get_default_download_dir(),
     "plugins_location": os.path.join(deluge.configmanager.get_config_dir(), "plugins"),
-    "state_location": os.path.join(deluge.configmanager.get_config_dir(), "state"),
     "prioritize_first_last_pieces": False,
     "random_port": True,
     "dht": True,
@@ -140,7 +137,10 @@ DEFAULT_PREFS = {
     "outgoing_ports": [0, 0],
     "random_outgoing_ports": True,
     "peer_tos": "0x00",
-    "rate_limit_ip_overhead": True
+    "rate_limit_ip_overhead": True,
+    "geoip_db_location": "/usr/share/GeoIP/GeoIP.dat",
+    "cache_size": 512,
+    "cache_expiry": 60
 }
 
 class PreferencesManager(component.Component):
@@ -152,16 +152,14 @@ class PreferencesManager(component.Component):
     def start(self):
         self.core = component.get("Core")
         self.session = component.get("Core").session
-        self.settings = component.get("Core").settings
-        self.signals = component.get("SignalManager")
 
         # Register set functions in the Config
         self.config.register_set_function("torrentfiles_location",
             self._on_set_torrentfiles_location)
-        self.config.register_set_function("state_location",
-            self._on_set_state_location)
         self.config.register_set_function("listen_ports",
             self._on_set_listen_ports)
+        self.config.register_set_function("listen_interface",
+            self._on_set_listen_interface)
         self.config.register_set_function("random_port",
             self._on_set_random_port)
         self.config.register_set_function("outgoing_ports",
@@ -220,12 +218,27 @@ class PreferencesManager(component.Component):
             self._on_new_release_check)
         self.config.register_set_function("rate_limit_ip_overhead",
             self._on_rate_limit_ip_overhead)
+        self.config.register_set_function("geoip_db_location",
+            self._on_geoip_db_location)
+        self.config.register_set_function("cache_size",
+            self._on_cache_size)
+        self.config.register_set_function("cache_expiry",
+            self._on_cache_expiry)
 
         self.config.register_change_callback(self._on_config_value_change)
 
+    def stop(self):
+        if self.new_release_timer:
+            self.new_release_timer.stop()
+
     # Config set functions
+    def session_set_setting(self, key, value):
+        settings = self.session.settings()
+        setattr(settings, key, value)
+        self.session.set_settings(settings)
+
     def _on_config_value_change(self, key, value):
-        self.signals.emit("config_value_changed", key, value)
+        component.get("EventManager").emit(ConfigValueChangedEvent(key, value))
 
     def _on_set_torrentfiles_location(self, key, value):
         if self.config["copy_torrent_file"]:
@@ -234,18 +247,15 @@ class PreferencesManager(component.Component):
             except Exception, e:
                 log.debug("Unable to make directory: %s", e)
 
-    def _on_set_state_location(self, key, value):
-        if not os.access(value, os.F_OK):
-            try:
-                os.makedirs(value)
-            except Exception, e:
-                log.debug("Unable to make directory: %s", e)
-
     def _on_set_listen_ports(self, key, value):
         # Only set the listen ports if random_port is not true
         if self.config["random_port"] is not True:
             log.debug("listen port range set to %s-%s", value[0], value[1])
-            self.session.listen_on(value[0], value[1])
+            self.session.listen_on(value[0], value[1], str(self.config["listen_interface"]))
+
+    def _on_set_listen_interface(self, key, value):
+        # Call the random_port callback since it'll do what we need
+        self._on_set_random_port("random_port", self.config["random_port"])
 
     def _on_set_random_port(self, key, value):
         log.debug("random port value set to %s", value)
@@ -263,13 +273,12 @@ class PreferencesManager(component.Component):
         # Set the listen ports
         log.debug("listen port range set to %s-%s", listen_ports[0],
             listen_ports[1])
-        self.session.listen_on(listen_ports[0], listen_ports[1])
+        self.session.listen_on(listen_ports[0], listen_ports[1], str(self.config["listen_interface"]))
 
     def _on_set_outgoing_ports(self, key, value):
         if not self.config["random_outgoing_ports"]:
             log.debug("outgoing port range set to %s-%s", value[0], value[1])
-            self.settings.outgoing_ports = value[0], value[1]
-            self.session.set_settings(self.settings)
+            self.session_set_setting("outgoing_ports", (value[0], value[1]))
 
     def _on_set_random_outgoing_ports(self, key, value):
         if value:
@@ -278,13 +287,11 @@ class PreferencesManager(component.Component):
     def _on_set_peer_tos(self, key, value):
         log.debug("setting peer_tos to: %s", value)
         try:
-            self.settings.peer_tos = chr(int(value, 16))
+            self.session_set_setting("peer_tos", chr(int(value, 16)))
         except ValueError, e:
             log.debug("Invalid tos byte: %s", e)
             return
 
-        self.session.set_settings(self.settings)
-
     def _on_set_dht(self, key, value):
         log.debug("dht value set to %s", value)
         state_file = deluge.configmanager.get_config_dir("dht.state")
@@ -381,51 +388,39 @@ class PreferencesManager(component.Component):
         self.session.set_max_half_open_connections(value)
 
     def _on_set_max_connections_per_second(self, key, value):
-        self.settings.connection_speed = value
-        self.session.set_settings(self.settings)
+        self.session_set_setting("connection_speed", value)
 
     def _on_ignore_limits_on_local_network(self, key, value):
-        self.settings.ignore_limits_on_local_network = value
-        self.session.set_settings(self.settings)
+        self.session_set_setting("ignore_limits_on_local_network", value)
 
     def _on_set_share_ratio_limit(self, key, value):
         log.debug("%s set to %s..", key, value)
-        self.settings.share_ratio_limit = value
-        self.session.set_settings(self.settings)
+        self.session_set_setting("share_ratio_limit", value)
 
     def _on_set_seed_time_ratio_limit(self, key, value):
         log.debug("%s set to %s..", key, value)
-        self.settings.seed_time_ratio_limit = value
-        self.session.set_settings(self.settings)
+        self.session_set_setting("seed_time_ratio_limit", value)
 
     def _on_set_seed_time_limit(self, key, value):
         log.debug("%s set to %s..", key, value)
         # This value is stored in minutes in deluge, but libtorrent wants seconds
-        self.settings.seed_time_limit = int(value * 60)
-        self.session.set_settings(self.settings)
+        self.session_set_setting("seed_time_limit", int(value * 60))
 
     def _on_set_max_active_downloading(self, key, value):
         log.debug("%s set to %s..", key, value)
-        log.debug("active_downloads: %s", self.settings.active_downloads)
-        self.settings.active_downloads = value
-        self.session.set_settings(self.settings)
+        self.session_set_setting("active_downloads", value)
 
     def _on_set_max_active_seeding(self, key, value):
         log.debug("%s set to %s..", key, value)
-        log.debug("active_seeds: %s", self.settings.active_seeds)
-        self.settings.active_seeds = value
-        self.session.set_settings(self.settings)
+        self.session_set_setting("active_seeds", value)
 
     def _on_set_max_active_limit(self, key, value):
         log.debug("%s set to %s..", key, value)
-        log.debug("active_limit: %s", self.settings.active_limit)
-        self.settings.active_limit = value
-        self.session.set_settings(self.settings)
+        self.session_set_setting("active_limit", value)
 
     def _on_set_dont_count_slow_torrents(self, key, value):
         log.debug("%s set to %s..", key, value)
-        self.settings.dont_count_slow_torrents = value
-        self.session.set_settings(self.settings)
+        self.session_set_setting("dont_count_slow_torrents", value)
 
     def _on_send_info(self, key, value):
         log.debug("Sending anonymous stats..")
@@ -448,7 +443,7 @@ class PreferencesManager(component.Component):
                             platform.machine() + "&python=" + platform.python_version() \
                             + "&deluge=" + deluge.common.get_version() \
                             + "&os=" + platform.system() \
-                            + "&plugins=" + quote_plus(self.config["enabled_plugins"])
+                            + "&plugins=" + quote_plus(":".join(self.config["enabled_plugins"]))
                         urlopen(url)
                     except IOError, e:
                         log.debug("Network error while trying to send info: %s", e)
@@ -462,27 +457,53 @@ class PreferencesManager(component.Component):
             log.debug("Checking for new release..")
             threading.Thread(target=self.core.get_new_release).start()
             if self.new_release_timer:
-                gobject.source_remove(self.new_release_timer)
+                self.new_release_timer.stop()
             # Set a timer to check for a new release every 3 days
-            self.new_release_timer = gobject.timeout_add(
-                72 * 60 * 60 * 1000, self._on_new_release_check, "new_release_check", True)
+            self.new_release_timer = LoopingCall(
+                self._on_new_release_check, "new_release_check", True)
+            self.new_release_timer.start(72 * 60 * 60, False)
         else:
             if self.new_release_timer:
-                gobject.source_remove(self.new_release_timer)
+                self.new_release_timer.stop()
 
     def _on_set_proxies(self, key, value):
         for k, v in value.items():
             if v["type"]:
                 proxy_settings = lt.proxy_settings()
                 proxy_settings.type = lt.proxy_type(v["type"])
-                proxy_settings.username = v["username"]
-                proxy_settings.password = v["password"]
-                proxy_settings.hostname = v["hostname"]
+                proxy_settings.username = str(v["username"])
+                proxy_settings.password = str(v["password"])
+                proxy_settings.hostname = str(v["hostname"])
                 proxy_settings.port = v["port"]
                 log.debug("setting %s proxy settings", k)
                 getattr(self.session, "set_%s_proxy" % k)(proxy_settings)
 
     def _on_rate_limit_ip_overhead(self, key, value):
         log.debug("%s: %s", key, value)
-        self.settings.rate_limit_ip_overhead = value
-        self.session.set_settings(self.settings)
+        self.session_set_setting("rate_limit_ip_overhead", value)
+
+    def _on_geoip_db_location(self, key, value):
+        log.debug("%s: %s", key, value)
+        # Load the GeoIP DB for country look-ups if available
+        geoip_db = ""
+        if os.path.exists(value):
+            geoip_db = value
+        elif os.path.exists(pkg_resources.resource_filename("deluge", os.path.join("data", "GeoIP.dat"))):
+            geoip_db = pkg_resources.resource_filename("deluge", os.path.join("data", "GeoIP.dat"))
+        else:
+            log.warning("Unable to find GeoIP database file!")
+
+        if geoip_db:
+            try:
+                self.session.load_country_db(str(geoip_db))
+            except Exception, e:
+                log.error("Unable to load geoip database!")
+                log.exception(e)
+
+    def _on_cache_size(self, key, value):
+        log.debug("%s: %s", key, value)
+        self.session_set_setting("cache_size", value)
+
+    def _on_cache_expiry(self, key, value):
+        log.debug("%s: %s", key, value)
+        self.session_set_setting("cache_expiry", value)
