@@ -31,17 +31,20 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
 import gtk
+import sys
 import pkg_resources
 import os.path
 import gobject
+import base64
 
-from deluge.ui.client import aclient as client
-import deluge.ui.gtkui.listview as listview
+from twisted.internet.threads import deferToThread
+
+from deluge.ui.client import client
+import listview
 import deluge.component as component
 import deluge.common
 from deluge.configmanager import ConfigManager
@@ -111,13 +114,24 @@ class CreateTorrentDialog:
 
         self.dialog.show()
 
+    def parse_piece_size_text(self, value):
+        psize, metric = value.split()
+        psize = int(psize)
+        if psize < 32:
+            # This is a MiB value
+            psize = psize * 1024 * 1024
+        else:
+            # This is a KiB value
+            psize = psize * 1024
+
+        return psize
+
     def adjust_piece_size(self):
         """Adjusts the recommended piece based on the file/folder/path selected."""
         size = self.files_treestore[0][2]
         model = self.glade.get_widget("combo_piece_size").get_model()
-        psize = 0
         for index,value in enumerate(model):
-            psize = int(value[0].split()[0]) * 1024
+            psize = self.parse_piece_size_text(value[0])
             pieces = size / psize
             if pieces < 2048 or (index + 1) == len(model):
                 self.glade.get_widget("combo_piece_size").set_active(index)
@@ -145,8 +159,10 @@ class CreateTorrentDialog:
             chooser.destroy()
             return
 
+        path = result.decode('utf-8').encode(sys.getfilesystemencoding())
+
         self.files_treestore.clear()
-        self.files_treestore.append(None, [result, gtk.STOCK_FILE, deluge.common.get_path_size(result)])
+        self.files_treestore.append(None, [result, gtk.STOCK_FILE, deluge.common.get_path_size(path)])
         self.adjust_piece_size()
         chooser.destroy()
 
@@ -171,9 +187,10 @@ class CreateTorrentDialog:
             chooser.destroy()
             return
 
-        self.files_treestore.clear()
+        path = result.decode('utf-8').encode(sys.getfilesystemencoding())
 
-        self.files_treestore.append(None, [result, gtk.STOCK_OPEN, deluge.common.get_path_size(result)])
+        self.files_treestore.clear()
+        self.files_treestore.append(None, [result, gtk.STOCK_OPEN, deluge.common.get_path_size(path)])
         self.adjust_piece_size()
         chooser.destroy()
 
@@ -194,10 +211,10 @@ class CreateTorrentDialog:
                     self.files_treestore.clear()
                     self.files_treestore.append(None, [result, gtk.STOCK_NETWORK, size])
                     self.adjust_piece_size()
-            client.get_path_size(_on_get_path_size, result)
+            client.core.get_path_size(result).addCallback(_on_get_path_size)
             client.force_call(True)
 
-        dialog.destroy()
+        dialog.hide()
 
     def _on_button_cancel_clicked(self, widget):
         log.debug("_on_button_cancel_clicked")
@@ -211,11 +228,14 @@ class CreateTorrentDialog:
         is_remote = self.files_treestore[0][1] == gtk.STOCK_NETWORK
         if is_remote:
             # This is a remote path
-            response = self.glade.get_widget("remote_save_dialog").run()
+            dialog = self.glade.get_widget("remote_save_dialog")
+            response = dialog.run()
             if response == gtk.RESPONSE_OK:
                 result = self.glade.get_widget("entry_save_path").get_text()
             else:
+                dialog.hide()
                 return
+            dialog.hide()
         else:
             # Setup the filechooserdialog
             chooser = gtk.FileChooserDialog(_("Save .torrent file"),
@@ -258,11 +278,22 @@ class CreateTorrentDialog:
         # Get the path
         path = self.files_treestore[0][0]
         # Get a list of trackers
-        trackers = [t[1] for t in self.trackers_liststore]
-        if len(trackers) == 0:
+        trackers = []
+        if not len(self.trackers_liststore):
             tracker = None
         else:
-            tracker = trackers[0]
+            # Create a list of lists [[tier0, ...], [tier1, ...], ...]
+            for tier, tracker in self.trackers_liststore:
+                try:
+                    tier_list = trackers[tier]
+                except IndexError:
+                    trackers.insert(tier, [])
+
+                trackers[tier].append(tracker)
+
+            # Get the first tracker in the first tier
+            tracker = trackers[0][0]
+
         # Get a list of webseeds
         webseeds = []
         b = self.glade.get_widget("textview_webseeds").get_buffer()
@@ -273,7 +304,8 @@ class CreateTorrentDialog:
                 webseeds.append(l)
         # Get the piece length in bytes
         combo = self.glade.get_widget("combo_piece_size")
-        piece_length = int(combo.get_model()[combo.get_active()][0].split()[0]) * 1024
+        piece_length = \
+            self.parse_piece_size_text(combo.get_model()[combo.get_active()][0])
         num_pieces = self.files_treestore[0][2] / piece_length
 
         author = self.glade.get_widget("entry_author").get_text()
@@ -282,16 +314,16 @@ class CreateTorrentDialog:
         add_to_session = self.glade.get_widget("chk_add_to_session").get_active()
 
         if is_remote:
-            client.create_torrent(
+            client.core.create_torrent(
                 path,
                 tracker,
                 piece_length,
                 comment,
                 result,
-                trackers,
+                webseeds,
                 private,
                 author,
-                webseeds,
+                trackers,
                 add_to_session)
 
         else:
@@ -299,26 +331,26 @@ class CreateTorrentDialog:
             self.glade.get_widget("progress_dialog").set_transient_for(component.get("MainWindow").window)
             self.glade.get_widget("progress_dialog").show_all()
 
-            import threading
-            threading.Thread(target=self.create_torrent,
-                args=(
-                    path,
+            def hide_progress(result):
+                self.glade.get_widget("progress_dialog").hide_all()
+
+            deferToThread(self.create_torrent,
+                    path.decode('utf-8'),
                     tracker,
                     piece_length,
                     self._on_create_torrent_progress,
                     comment,
-                    result,
-                    trackers,
+                    result.decode('utf-8'),
+                    webseeds,
                     private,
                     author,
-                    webseeds,
-                    add_to_session)).start()
+                    trackers,
+                    add_to_session).addCallback(hide_progress)
 
-        chooser.destroy()
         self.dialog.destroy()
 
     def create_torrent(self, path, tracker, piece_length, progress, comment, target,
-                        url_list, private, created_by, httpseeds, add_to_session):
+                        webseeds, private, created_by, trackers, add_to_session):
         import deluge.metafile
         deluge.metafile.make_meta_file(
             path,
@@ -327,17 +359,16 @@ class CreateTorrentDialog:
             progress=progress,
             comment=comment,
             target=target,
-            url_list=url_list,
+            webseeds=webseeds,
             private=private,
             created_by=created_by,
-            httpseeds=httpseeds)
+            trackers=trackers)
 
-        def finish_up(target):
-            self.glade.get_widget("progress_dialog").hide_all()
-            if add_to_session:
-                client.add_torrent_file([target])
-
-        gobject.idle_add(finish_up, target)
+        if add_to_session:
+            client.core.add_torrent_file(
+                os.path.split(target)[-1],
+                base64.encodestring(open(target).read()),
+                {"download_location": os.path.split(path)[0]})
 
     def _on_create_torrent_progress(self, value, num_pieces):
         percent = float(value)/float(num_pieces)
@@ -368,7 +399,10 @@ class CreateTorrentDialog:
         dialog = glade.get_widget("add_tracker_dialog")
         dialog.set_transient_for(self.dialog)
         textview = glade.get_widget("textview_trackers")
-        textview.get_buffer().set_text("")
+        if self.config["createtorrent.trackers"]:
+            textview.get_buffer().set_text("\n".join(self.config["createtorrent.trackers"]))
+        else:
+            textview.get_buffer().set_text("")
         textview.grab_focus()
         response = dialog.run()
 
@@ -377,6 +411,8 @@ class CreateTorrentDialog:
             trackers = []
             b = textview.get_buffer()
             lines = b.get_text(b.get_start_iter(), b.get_end_iter()).strip().split("\n")
+            self.config["createtorrent.trackers"] = lines
+            log.debug("lines: %s", lines)
             for l in lines:
                 if deluge.common.is_url(l):
                     trackers.append(l)
