@@ -79,10 +79,6 @@ def cell_filename(column, cell, model, row, data):
     cell.set_property("text", os.path.split(filepath)[1])
 
 def cell_progress(column, cell, model, row, data):
-    if model.get_value(row, 5) == -1:
-    # This is a folder, so lets just set it blank for now
-        cell.set_property("visible", False)
-        return
     text = model.get_value(row, data[0])
     value = model.get_value(row, data[1])
     cell.set_property("visible", True)
@@ -234,8 +230,8 @@ class FilesTab(Tab):
         # Setup state dict
         state = {
             "columns": {},
-            "sort_id": column_id,
-            "sort_order": int(sort_order) if sort_order else None
+            "sort_id": int(column_id) if column_id >= 0 else None,
+            "sort_order": int(sort_order) if sort_order >= 0 else None
         }
 
         for index, column in enumerate(self.listview.get_columns()):
@@ -272,7 +268,7 @@ class FilesTab(Tab):
         if state == None:
             return
 
-        if state["sort_id"] and state["sort_order"] is not None:
+        if state["sort_id"] is not None and state["sort_order"] is not None:
             self.treestore.set_sort_column_id(state["sort_id"], state["sort_order"])
 
         for (index, column) in enumerate(self.listview.get_columns()):
@@ -319,7 +315,7 @@ class FilesTab(Tab):
             log.debug("Getting file list from core..")
             status_keys += ["files"]
 
-        client.core.get_torrent_status(self.torrent_id, status_keys).addCallback(self._on_get_torrent_status)
+        component.get("SessionProxy").get_torrent_status(self.torrent_id, status_keys).addCallback(self._on_get_torrent_status)
 
     def clear(self):
         self.treestore.clear()
@@ -327,7 +323,7 @@ class FilesTab(Tab):
 
     def _on_row_activated(self, tree, path, view_column):
         if client.is_localhost:
-            client.core.get_torrent_status(self.torrent_id, ["save_path", "files"]).addCallback(self._on_open_file)
+            component.get("SessionProxy").get_torrent_status(self.torrent_id, ["save_path", "files"]).addCallback(self._on_open_file)
 
     def get_file_path(self, row, path=""):
         if not row:
@@ -419,6 +415,33 @@ class FilesTab(Tab):
             self.get_files_from_tree(row.iterchildren(), files_list, indent+1)
         return None
 
+    def update_folder_percentages(self):
+        """
+        Go through the tree and update the folder complete percentages.
+        """
+
+        root = self.treestore.get_iter_root()
+        if self.treestore[root][5] != -1:
+            return
+
+        def get_completed_bytes(row):
+            bytes = 0
+            parent = self.treestore.iter_parent(row)
+            while row:
+                if self.treestore.iter_children(row):
+                    bytes += get_completed_bytes(self.treestore.iter_children(row))
+                else:
+                    bytes += self.treestore[row][1] * (float(self.treestore[row][3]) / 100.0)
+
+                row = self.treestore.iter_next(row)
+
+            value = (float(bytes) / float(self.treestore[parent][1])) * 100
+            self.treestore[parent][3] = value
+            self.treestore[parent][2] = "%.2f%%" % value
+            return bytes
+
+        get_completed_bytes(self.treestore.iter_children(root))
+
     def _on_get_torrent_status(self, status):
         # Store this torrent's compact setting
         if "compact" in status:
@@ -434,9 +457,8 @@ class FilesTab(Tab):
         files_list.sort()
         for index, row in files_list:
             # Do not update a row that is being edited
-            if self._editing_index is not None:
-                if self._editing_index == row[5]:
-                    continue
+            if self._editing_index == row[5]:
+                continue
 
             progress_string = "%.2f%%" % (status["file_progress"][index] * 100)
             if row[2] != progress_string:
@@ -447,6 +469,9 @@ class FilesTab(Tab):
             file_priority = status["file_priorities"][index]
             if row[4] != file_priority:
                 row[4] = file_priority
+        if self._editing_index != -1:
+            # Only update if no folder is being edited
+            self.update_folder_percentages()
 
     def _on_button_press_event(self, widget, event):
         """This is a callback for showing the right-click context menu."""
