@@ -31,16 +31,15 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
 import deluge.component as component
 import deluge.common
-import deluge.ui.gtkui.common as common
+import common
 from deluge.log import LOG as log
 from deluge.configmanager import ConfigManager
-from deluge.ui.client import aclient as client
+from deluge.ui.client import client
 
 class Notification:
     def __init__(self):
@@ -54,7 +53,7 @@ class Notification:
             self.get_torrent_status(torrent_id)
 
     def get_torrent_status(self, torrent_id):
-        client.get_torrent_status(self._on_get_torrent_status, torrent_id, ["name", "num_files", "total_payload_download"])
+        component.get("SessionProxy").get_torrent_status(torrent_id, ["name", "num_files", "total_payload_download"]).addCallback(self._on_get_torrent_status)
 
     def _on_get_torrent_status(self, status):
         if status is None:
@@ -75,12 +74,14 @@ class Notification:
             except:
                 log.warning("pynotify is not installed")
             else:
-                if pynotify.init("Deluge"):
-                    self.note = pynotify.Notification(_("Torrent complete"),
-                        status["name"] + "\n" + _("Including %i files" % status["num_files"]))
-                    self.note.set_icon_from_pixbuf(common.get_logo(48))
-                    if not self.note.show():
-                        log.warning("pynotify failed to show notification")
+                if not pynotify.init("Deluge"):
+                    return
+                title = deluge.common.xml_encode(_("Torrent complete"))
+                message = deluge.common.xml_encode(status["name"] + "\n" + _("Including %i files" % status["num_files"]))
+                self.note = pynotify.Notification(title, message)
+                self.note.set_icon_from_pixbuf(common.get_logo(48))
+                if not self.note.show():
+                    log.warning("pynotify failed to show notification")
 
     def sound(self):
         """plays a sound when a torrent finishes"""
@@ -105,9 +106,9 @@ class Notification:
         headers = "From: %s\r\nTo: %s\r\nSubject: %s\r\n\r\n" % (
             self.config["ntf_email_add"], self.config["ntf_email_add"],
                 "Finished torrent %s" % (status["name"]))
-        text = _("This email is to inform you that Deluge has finished downloading %s , \
-            which includes %i files.\nTo stop receiving these alerts, simply turn off \
-            email notification in Deluge's preferences.\n\nThank you,\nDeluge") % (status["name"], status["num_files"])
+        text = _("This email is to inform you that Deluge has finished downloading %(name)s , \
+            which includes %(num_files)i files.\nTo stop receiving these alerts, simply turn off \
+            email notification in Deluge's preferences.\n\nThank you,\nDeluge") % {"name": status["name"], "num_files": status["num_files"]}
         message = headers + text
         if self.config["ntf_security"] == 'SSL':
             port = 465
