@@ -17,9 +17,9 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with deluge.    If not, write to:
-# 	The Free Software Foundation, Inc.,
-# 	51 Franklin Street, Fifth Floor
-# 	Boston, MA  02110-1301, USA.
+#     The Free Software Foundation, Inc.,
+#     51 Franklin Street, Fifth Floor
+#     Boston, MA  02110-1301, USA.
 #
 #    In addition, as a special exception, the copyright holders give
 #    permission to link the code of portions of this program with the OpenSSL
@@ -72,8 +72,8 @@ class Preferences(component.Component):
         self.treeview.append_column(column)
         # Add the default categories
         i = 0
-        for category in [_("Downloads"), _("Network"), _("Bandwidth"), _("Interface"),
-            _("Other"), _("Daemon"), _("Queue"), _("Proxy"), _("Notification"),
+        for category in [_("Downloads"), _("Network"), _("Bandwidth"),
+            _("Interface"), _("Other"), _("Daemon"), _("Queue"), _("Proxy"),
             _("Cache"), _("Plugins")]:
             self.liststore.append([i, category])
             i += 1
@@ -173,8 +173,13 @@ class Preferences(component.Component):
         if self.iter_to_remove != None:
             self.liststore.remove(self.iter_to_remove)
 
+        # We need to re-adjust the index values for the remaining pages
+        for i, (index, name) in enumerate(self.liststore):
+            self.liststore[i][0] = i
+
     def show(self, page=None):
-        """Page should be the string in the left list.. ie, 'Network' or 'Bandwidth'"""
+        """Page should be the string in the left list.. ie, 'Network' or
+        'Bandwidth'"""
         if page != None:
             for (index, string) in self.liststore:
                 if page == string:
@@ -222,6 +227,8 @@ class Preferences(component.Component):
                     ("filename", self.core_config["move_completed_path"]),
                 "chk_copy_torrent_file": \
                     ("active", self.core_config["copy_torrent_file"]),
+                "chk_del_copy_torrent_file": \
+                    ("active", self.core_config["del_copy_torrent_file"]),
                 "torrent_files_button": \
                     ("filename", self.core_config["torrentfiles_location"]),
                 "chk_autoadd": \
@@ -374,6 +381,7 @@ class Preferences(component.Component):
                 "chk_move_completed",
                 "move_completed_path_button",
                 "chk_copy_torrent_file",
+                "chk_del_copy_torrent_file",
                 "torrent_files_button",
                 "chk_autoadd",
                 "folder_autoadd",
@@ -471,34 +479,6 @@ class Preferences(component.Component):
         self.glade.get_widget("chk_show_new_releases").set_active(
             self.gtkui_config["show_new_releases"])
 
-        ## Notification tab ##
-        self.glade.get_widget("chk_ntf_tray_blink").set_active(
-            self.gtkui_config["ntf_tray_blink"])
-        if deluge.common.windows_check():
-            self.glade.get_widget("chk_ntf_popup").set_sensitive(False)
-        else:
-            self.glade.get_widget("chk_ntf_popup").set_active(
-                self.gtkui_config["ntf_popup"])
-        self.glade.get_widget("chk_ntf_email").set_active(
-            self.gtkui_config["ntf_email"])
-        self.glade.get_widget("chk_ntf_sound").set_active(
-            self.gtkui_config["ntf_sound"])
-        if self.gtkui_config["ntf_sound_path"]:
-            self.glade.get_widget("combo_ntf_sound_path").set_filename(self.gtkui_config["ntf_sound_path"])
-        self.glade.get_widget("txt_ntf_email").set_text(
-            self.gtkui_config["ntf_email_add"])
-        self.glade.get_widget("txt_ntf_server").set_text(
-            self.gtkui_config["ntf_server"])
-        self.glade.get_widget("txt_ntf_username").set_text(
-            self.gtkui_config["ntf_username"])
-        self.glade.get_widget("txt_ntf_pass").set_text(
-            self.gtkui_config["ntf_pass"])
-        if not self.gtkui_config["ntf_security"]:
-            self.glade.get_widget("rad_ntf_none").set_active(True)
-        elif self.gtkui_config["ntf_security"] == 'SSL':
-            self.glade.get_widget("rad_ntf_none").set_active(True)
-        elif self.gtkui_config["ntf_security"] == 'TLS':
-            self.glade.get_widget("rad_ntf_tls").set_active(True)
 
         ## Cache tab ##
         if client.connected():
@@ -544,6 +524,8 @@ class Preferences(component.Component):
             self.glade.get_widget("chk_focus_dialog").get_active()
         new_core_config["copy_torrent_file"] = \
             self.glade.get_widget("chk_copy_torrent_file").get_active()
+        new_core_config["del_copy_torrent_file"] = \
+            self.glade.get_widget("chk_del_copy_torrent_file").get_active()
         new_core_config["move_completed"] = \
             self.glade.get_widget("chk_move_completed").get_active()
         if client.is_localhost():
@@ -579,19 +561,17 @@ class Preferences(component.Component):
             self.glade.get_widget("chk_add_paused").get_active()
 
         ## Network tab ##
-        listen_ports = []
-        listen_ports.append(
-            self.glade.get_widget("spin_port_min").get_value_as_int())
-        listen_ports.append(
-            self.glade.get_widget("spin_port_max").get_value_as_int())
+        listen_ports = (
+            self.glade.get_widget("spin_port_min").get_value_as_int(),
+            self.glade.get_widget("spin_port_max").get_value_as_int()
+        )
         new_core_config["listen_ports"] = listen_ports
         new_core_config["random_port"] = \
             self.glade.get_widget("chk_random_port").get_active()
-        outgoing_ports = []
-        outgoing_ports.append(
-            self.glade.get_widget("spin_outgoing_port_min").get_value_as_int())
-        outgoing_ports.append(
-            self.glade.get_widget("spin_outgoing_port_max").get_value_as_int())
+        outgoing_ports = (
+            self.glade.get_widget("spin_outgoing_port_min").get_value_as_int(),
+            self.glade.get_widget("spin_outgoing_port_max").get_value_as_int()
+        )
         new_core_config["outgoing_ports"] = outgoing_ports
         new_core_config["random_outgoing_ports"] = \
             self.glade.get_widget("chk_random_outgoing_ports").get_active()
@@ -665,32 +645,6 @@ class Preferences(component.Component):
         new_gtkui_config["show_rate_in_title"] = \
             self.glade.get_widget("chk_show_rate_in_title").get_active()
 
-        ## Notification tab ##
-        new_gtkui_config["ntf_tray_blink"] = \
-            self.glade.get_widget("chk_ntf_tray_blink").get_active()
-        new_gtkui_config["ntf_popup"] = \
-            self.glade.get_widget("chk_ntf_popup").get_active()
-        new_gtkui_config["ntf_sound"] = \
-            self.glade.get_widget("chk_ntf_sound").get_active()
-        new_gtkui_config["ntf_email"] = \
-            self.glade.get_widget("chk_ntf_email").get_active()
-        new_gtkui_config["ntf_email_add"] = \
-            self.glade.get_widget("txt_ntf_email").get_text()
-        new_gtkui_config["ntf_username"] = \
-            self.glade.get_widget("txt_ntf_username").get_text()
-        new_gtkui_config["ntf_pass"] = \
-            self.glade.get_widget("txt_ntf_pass").get_text()
-        new_gtkui_config["ntf_server"] = \
-            self.glade.get_widget("txt_ntf_server").get_text()
-        new_gtkui_config["ntf_sound_path"] = \
-                self.glade.get_widget("combo_ntf_sound_path").get_filename()
-        if self.glade.get_widget("rad_ntf_none").get_active():
-            new_gtkui_config["ntf_security"] = None
-        elif self.glade.get_widget("rad_ntf_ssl").get_active():
-            new_gtkui_config["ntf_security"] = 'SSL'
-        elif self.glade.get_widget("rad_ntf_tls").get_active():
-            new_gtkui_config["ntf_security"] = 'TLS'
-
         ## Other tab ##
         new_gtkui_config["show_new_releases"] = \
             self.glade.get_widget("chk_show_new_releases").get_active()
@@ -770,11 +724,12 @@ class Preferences(component.Component):
                 if self.core_config[key] != new_core_config[key]:
                     config_to_set[key] = new_core_config[key]
 
-            # Set each changed config value in the core
-            client.core.set_config(config_to_set)
-            client.force_call(True)
-            # Update the configuration
-            self.core_config.update(config_to_set)
+            if config_to_set:
+                # Set each changed config value in the core
+                client.core.set_config(config_to_set)
+                client.force_call(True)
+                # Update the configuration
+                self.core_config.update(config_to_set)
 
         if hide:
             self.hide()
@@ -783,6 +738,7 @@ class Preferences(component.Component):
             self.show()
 
     def hide(self):
+        self.glade.get_widget("port_img").hide()
         self.pref_dialog.hide()
 
     def __update_cache_status(self):
@@ -829,7 +785,8 @@ class Preferences(component.Component):
                 "radio_open_folder_custom": {"combo_file_manager": False,
                                              "txt_open_folder_location": True},
                 "chk_move_completed" : {"move_completed_path_button" : True},
-                "chk_copy_torrent_file" : {"torrent_files_button" : True},
+                "chk_copy_torrent_file" : {"torrent_files_button" : True,
+                                           "chk_del_copy_torrent_file" : True},
                 "chk_autoadd" : {"folder_autoadd" : True},
                 "chk_seed_ratio" : {"spin_share_ratio": True,
                                     "chk_remove_ratio" : True}
@@ -881,6 +838,10 @@ class Preferences(component.Component):
                 self.glade.get_widget("port_img").set_from_stock(gtk.STOCK_DIALOG_WARNING, 4)
                 self.glade.get_widget("port_img").show()
         client.core.test_listen_port().addCallback(on_get_test)
+        self.glade.get_widget("port_img").set_from_file(
+            deluge.common.get_pixmap('loading.gif')
+        )
+        self.glade.get_widget("port_img").show()
         client.force_call()
 
     def on_plugin_toggled(self, renderer, path):
