@@ -17,9 +17,9 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with deluge.    If not, write to:
-# 	The Free Software Foundation, Inc.,
-# 	51 Franklin Street, Fifth Floor
-# 	Boston, MA  02110-1301, USA.
+#     The Free Software Foundation, Inc.,
+#     51 Franklin Street, Fifth Floor
+#     Boston, MA  02110-1301, USA.
 #
 #    In addition, as a special exception, the copyright holders give
 #    permission to link the code of portions of this program with the OpenSSL
@@ -87,19 +87,29 @@ def cell_data_statusicon(column, cell, model, row, data):
         pass
 
 def cell_data_trackericon(column, cell, model, row, data):
-    icon_path = component.get("TrackerIcons").get(model[row][data])
-    if icon_path:
-        try:
-            icon = gtk.gdk.pixbuf_new_from_file_at_size(icon_path, 16, 16)
-        except Exception, e:
-            pass
+    def on_get_icon(icon):
+        def create_blank_icon():
+            i = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 16, 16)
+            i.fill(0x00000000)
+            return i
+
+        if icon:
+            try:
+                icon = gtk.gdk.pixbuf_new_from_file_at_size(icon.get_filename(), 16, 16)
+            except Exception, e:
+                icon = create_blank_icon()
+        else:
+            icon = create_blank_icon()
+
+        if cell.get_property("pixbuf") != icon:
+            cell.set_property("pixbuf", icon)
+
+    host = model[row][data]
+    if host:
+        d = component.get("TrackerIcons").get(host)
+        d.addCallback(on_get_icon)
     else:
-        icon = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 16, 16)
-        icon.fill(0x00000000)
-
-    if cell.get_property("pixbuf") != icon:
-        cell.set_property("pixbuf", icon)
-
+        on_get_icon(None)
 
 def cell_data_progress(column, cell, model, row, data):
     """Display progress bar with text"""
@@ -120,9 +130,7 @@ def cell_data_queue(column, cell, model, row, data):
     else:
         cell.set_property("text", value + 1)
 
-def queue_column_sort(model, iter1, iter2, data):
-    v1 = model[iter1][data]
-    v2 = model[iter2][data]
+def queue_peer_seed_sort_function(v1, v2):
     if v1 == v2:
         return 0
     if v2 < 0:
@@ -134,6 +142,12 @@ def queue_column_sort(model, iter1, iter2, data):
     if v2 > v1:
         return -1
 
+def queue_column_sort(model, iter1, iter2, data):
+    v1 = model[iter1][data]
+    v2 = model[iter2][data]
+    return queue_peer_seed_sort_function(v1, v2)
+
+
 def eta_column_sort(model, iter1, iter2, data):
     v1 = model[iter1][data]
     v2 = model[iter2][data]
@@ -148,10 +162,19 @@ def eta_column_sort(model, iter1, iter2, data):
     if v2 > v1:
         return -1
 
+def seed_peer_column_sort(model, iter1, iter2, data):
+    v1 = model[iter1][data]         # num seeds/peers
+    v3 = model[iter2][data]         # num seeds/peers
+    if v1 == v3:
+        v2 = model[iter1][data+1]   # total seeds/peers
+        v4 = model[iter2][data+1]   # total seeds/peers
+        return queue_peer_seed_sort_function(v2, v4)
+    return queue_peer_seed_sort_function(v1, v3)
+
 class TorrentView(listview.ListView, component.Component):
     """TorrentView handles the listing of torrents."""
     def __init__(self):
-        component.Component.__init__(self, "TorrentView", interval=2)
+        component.Component.__init__(self, "TorrentView", interval=2, depend=["SessionProxy"])
         self.window = component.get("MainWindow")
         # Call the ListView constructor
         listview.ListView.__init__(self,
@@ -159,6 +182,9 @@ class TorrentView(listview.ListView, component.Component):
                             "torrentview.state")
         log.debug("TorrentView Init..")
 
+        # If we have gotten the state yet
+        self.got_state = False
+
         # This is where status updates are put
         self.status = {}
 
@@ -173,54 +199,42 @@ class TorrentView(listview.ListView, component.Component):
         # Add the columns to the listview
         self.add_text_column("torrent_id", hidden=True)
         self.add_bool_column("dirty", hidden=True)
-        self.add_func_column("#", cell_data_queue, [int], status_field=["queue"], sort_func=queue_column_sort)
-        self.add_texticon_column(_("Name"), status_field=["state", "name"],
-                                            function=cell_data_statusicon)
-        self.add_func_column(_("Size"),
-                                            listview.cell_data_size,
-                                            [gobject.TYPE_UINT64],
-                                            status_field=["total_wanted"])
+        self.add_func_column("#", cell_data_queue, [int],
+                             status_field=["queue"],
+                             sort_func=queue_column_sort)
+        self.add_texticon_column(_("Name"),
+                                 status_field=["state", "name"],
+                                 function=cell_data_statusicon)
+        self.add_func_column(_("Size"), listview.cell_data_size,
+                             [gobject.TYPE_UINT64],
+                             status_field=["total_wanted"])
         self.add_progress_column(_("Progress"),
-                                    status_field=["progress", "state"],
-                                    col_types=[float, str],
-                                    function=cell_data_progress)
-        self.add_func_column(_("Seeders"),
-                                        listview.cell_data_peer,
-                                        [int, int],
-                                        status_field=["num_seeds",
-                                                        "total_seeds"])
-        self.add_func_column(_("Peers"),
-                                        listview.cell_data_peer,
-                                        [int, int],
-                                        status_field=["num_peers",
-                                                        "total_peers"])
-        self.add_func_column(_("Down Speed"),
-                                        listview.cell_data_speed,
-                                        [float],
-                                        status_field=["download_payload_rate"])
-        self.add_func_column(_("Up Speed"),
-                                        listview.cell_data_speed,
-                                        [float],
-                                        status_field=["upload_payload_rate"])
-        self.add_func_column(_("ETA"),
-                                            listview.cell_data_time,
-                                            [int],
-                                            status_field=["eta"],
-                                            sort_func=eta_column_sort)
-        self.add_func_column(_("Ratio"),
-                                            listview.cell_data_ratio,
-                                            [float],
-                                            status_field=["ratio"])
-        self.add_func_column(_("Avail"),
-                                            listview.cell_data_ratio,
-                                            [float],
-                                            status_field=["distributed_copies"])
-        self.add_func_column(_("Added"),
-                                            listview.cell_data_date,
-                                            [float],
-                                            status_field=["time_added"])
-        self.add_texticon_column(_("Tracker"), status_field=["tracker_host", "tracker_host"],
-                                    function=cell_data_trackericon)
+                                 status_field=["progress", "state"],
+                                 col_types=[float, str],
+                                 function=cell_data_progress)
+        self.add_func_column(_("Seeders"), listview.cell_data_peer, [int, int],
+                             status_field=["num_seeds", "total_seeds"],
+                             sort_func=seed_peer_column_sort)
+        self.add_func_column(_("Peers"), listview.cell_data_peer, [int, int],
+                             status_field=["num_peers", "total_peers"],
+                             sort_func=seed_peer_column_sort)
+        self.add_func_column(_("Down Speed"), listview.cell_data_speed, [float],
+                             status_field=["download_payload_rate"])
+        self.add_func_column(_("Up Speed"), listview.cell_data_speed, [float],
+                             status_field=["upload_payload_rate"])
+        self.add_func_column(_("ETA"), listview.cell_data_time, [int],
+                             status_field=["eta"], sort_func=eta_column_sort)
+        self.add_func_column(_("Ratio"), listview.cell_data_ratio, [float],
+                             status_field=["ratio"])
+        self.add_func_column(_("Avail"), listview.cell_data_ratio, [float],
+                             status_field=["distributed_copies"])
+        self.add_func_column(_("Added"), listview.cell_data_date, [float],
+                             status_field=["time_added"])
+        self.add_texticon_column(_("Tracker"),
+                                 status_field=["tracker_host", "tracker_host"],
+                                 function=cell_data_trackericon)
+
+        self.add_text_column(_("Save Path"), status_field=["save_path"])
 
         # Set filter to None for now
         self.filter = None
@@ -239,7 +253,7 @@ class TorrentView(listview.ListView, component.Component):
                                     self.on_selection_changed)
 
         self.treeview.connect("drag-drop", self.on_drag_drop)
-        self.treeview.connect("key_press_event", self.on_key_press_event)
+        self.treeview.connect("key-press-event", self.on_key_press_event)
 
         client.register_event_handler("TorrentStateChangedEvent", self.on_torrentstatechanged_event)
         client.register_event_handler("TorrentAddedEvent", self.on_torrentadded_event)
@@ -252,10 +266,9 @@ class TorrentView(listview.ListView, component.Component):
         """Start the torrentview"""
         # We need to get the core session state to know which torrents are in
         # the session so we can add them to our list.
-        client.core.get_session_state().addCallback(self._on_session_state)
+        component.get("SessionProxy").get_torrents_status({}, []).addCallback(self._on_session_state)
 
     def _on_session_state(self, state):
-        log.debug("on_session_state: %s", state)
         self.treeview.freeze_child_notify()
         model = self.treeview.get_model()
         for torrent_id in state:
@@ -263,7 +276,11 @@ class TorrentView(listview.ListView, component.Component):
             self.mark_dirty(torrent_id)
         self.treeview.set_model(model)
         self.treeview.thaw_child_notify()
-        self.update()
+        self.got_state = True
+        # Update the view right away with our status
+        self.status = state
+        self.set_columns_to_update()
+        self.update_view()
 
     def stop(self):
         """Stops the torrentview"""
@@ -289,10 +306,8 @@ class TorrentView(listview.ListView, component.Component):
         self.filter = dict(filter_dict) #copied version of filter_dict.
         self.update()
 
-    def send_status_request(self, columns=None):
-        # Store the 'status_fields' we need to send to core
+    def set_columns_to_update(self, columns=None):
         status_keys = []
-        # Store the actual columns we will be updating
         self.columns_to_update = []
 
         if columns is None:
@@ -312,6 +327,11 @@ class TorrentView(listview.ListView, component.Component):
 
         # Remove duplicate keys
         self.columns_to_update = list(set(self.columns_to_update))
+        return status_keys
+
+    def send_status_request(self, columns=None):
+        # Store the 'status_fields' we need to send to core
+        status_keys = self.set_columns_to_update(columns)
 
         # If there is nothing in status_keys then we must not continue
         if status_keys is []:
@@ -322,12 +342,13 @@ class TorrentView(listview.ListView, component.Component):
 
         # Request the statuses for all these torrent_ids, this is async so we
         # will deal with the return in a signal callback.
-        client.core.get_torrents_status(
+        component.get("SessionProxy").get_torrents_status(
             self.filter, status_keys).addCallback(self._on_get_torrents_status)
 
     def update(self):
-        # Send a status request
-        gobject.idle_add(self.send_status_request)
+        if self.got_state:
+            # Send a status request
+            gobject.idle_add(self.send_status_request)
 
     def update_view(self, columns=None):
         """Update the view.  If columns is not None, it will attempt to only
@@ -352,13 +373,14 @@ class TorrentView(listview.ListView, component.Component):
                 for column in self.columns_to_update:
                     column_index = self.get_column_index(column)
                     for i, status_field in enumerate(self.columns[column].status_field):
-                        try:
-                            # Only update if different
-                            row_value = status[torrent_id][status_field]
-                            if row[column_index[i]] != row_value:
-                                row[column_index[i]] = row_value
-                        except Exception, e:
-                            log.debug("%s", e)
+                        if status_field in status[torrent_id]:
+                            try:
+                                # Only update if different
+                                row_value = status[torrent_id][status_field]
+                                if row[column_index[i]] != row_value:
+                                    row[column_index[i]] = row_value
+                            except Exception, e:
+                                log.debug("%s", e)
 
         component.get("MenuBar").update_menu()
 
@@ -480,18 +502,6 @@ class TorrentView(listview.ListView, component.Component):
             torrentmenu.popup(None, None, None, event.button, event.time)
             return True
 
-    def on_key_press_event(self, widget, event):
-        # Menu key
-        if gtk.gdk.keyval_name(event.keyval) != "Menu":
-            return
-
-        if not self.get_selected_torrent():
-            return
-
-        torrentmenu = component.get("MenuBar").torrentmenu
-        torrentmenu.popup(None, None, None, 3, event.time)
-        return True
-
     def on_selection_changed(self, treeselection):
         """This callback is know when the selection has changed."""
         log.debug("on_selection_changed")
@@ -534,10 +544,19 @@ class TorrentView(listview.ListView, component.Component):
         keyname = gtk.gdk.keyval_name(event.keyval)
         func = getattr(self, 'keypress_' + keyname, None)
         if func:
-            return func()
+            return func(event)
 
-    def keypress_Delete(self):
+    def keypress_Delete(self, event):
         log.debug("keypress_Delete")
         torrents = self.get_selected_torrents()
         if torrents:
             RemoveTorrentDialog(torrents).run()
+
+    def keypress_Menu(self, event):
+        log.debug("keypress_Menu")
+        if not self.get_selected_torrent():
+            return
+
+        torrentmenu = component.get("MenuBar").torrentmenu
+        torrentmenu.popup(None, None, None, 3, event.time)
+        return True
